import { ImplementableApi } from './implementableApi';

namespace PeerTubeRequester {
    export type Config = ImplementableApi.Config & {
        domain_name: string;
        username: string;
        password: string;
    };
}

type UploadInstruction = {
    [key: string]: string;
    channelID: string;
    targetUrl: string;
    name: string;
    description: string;
    originallyPublishedAt: string;
    thumbnailsfile: string;
};

type ClientToken = {
    client_id: string;
    client_secret: string;
    grant_type: 'password';
    response_type: 'code';
    username: string;
    password: string;
};

type UserToken = {
    access_token: string;
    token_type: string;
    expires_in: string;
    refresh_token: string;
};

class PeerTubeRequester extends ImplementableApi {
    readonly domain_name: string;
    readonly username: string;
    readonly password: string;

    constructor(readonly config: PeerTubeRequester.Config) {
        super(config);
        this.domain_name = config.domain_name;
        this.username = config.username;
        this.password = config.password;
    }

    public receivedMessage(message: ImplementableApi.Message) {
        switch (message.type) {
            case 'newEntriesNotify':
                this.newEntriesNotify(message);
                break;
            default:
                break;
        }
    }

    private newEntriesNotify(message: ImplementableApi.Message) {
        // parse content
        const items = message.rawContent['items'];
        if (Array.isArray(items))
            items.forEach((item) => {
                const media_group = item['media:group'];
                const args: UploadInstruction = {
                    channelID: 'undefined', // to do binding avec les idDeChaines Skeptikom
                    description: media_group['media:description'][0],
                    name: media_group['media:title'][0],
                    originallyPublishedAt: item.pubDate,
                    targetUrl: media_group['media:content'][0]['$']['url'],
                    thumbnailsfile:
                        media_group['media:thumbnail'][0]['$']['url'],
                };
                this.apiRequest(args);
            });
    }

    private async apiRequest(message: UploadInstruction) {
        // Auth
        const client_info: ClientToken = {
            ...(await (
                await fetch(
                    `https://${this.domain_name}/api/v1/oauth-clients/local`
                )
            ).json()),
            ...{
                grant_type: 'password',
                response_type: 'code',
                username: this.username,
                password: this.password,
            },
        };

        const myAuthForm = new FormData();
        for (const key in client_info) myAuthForm.append(key, message[key]);

        const tokens_info: UserToken = await (
            await fetch(`https://${this.domain_name}/api/v1/users/token`, {
                method: 'get',
                body: myAuthForm,
            })
        ).json();

        // Upload
        const myUploadForm = new FormData();
        const myHeader = new Headers();
        myHeader.append('Authorization', `Bearer ${tokens_info.access_token}`);
        for (const key in message) myUploadForm.append(key, message[key]);
        await fetch(`https://${this.domain_name}/api/v1/videos/imports`, {
            method: 'post',
            headers: myHeader,
            body: myUploadForm,
        });
    }
}

export { PeerTubeRequester };
